"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var openvidu_browser_1 = require("openvidu-browser");
var angular_bigscreen_1 = require("angular-bigscreen");
var OpenViduComponent = (function () {
    function OpenViduComponent(renderer, bigScreenService) {
        this.renderer = renderer;
        this.bigScreenService = bigScreenService;
        // Outputs
        this.onRoomConnected = new core_1.EventEmitter();
        this.onRoomClosed = new core_1.EventEmitter();
        this.onLostConnection = new core_1.EventEmitter();
        //@Output() onErrorRoom: EventEmitter<any> = new EventEmitter();
        //@Output() onStreamAdded: EventEmitter<any> = new EventEmitter();
        //@Output() onStreamRemoved: EventEmitter<any> = new EventEmitter();
        this.onParticipantJoined = new core_1.EventEmitter();
        //@Output() onParticpantPublished: EventEmitter<any> = new EventEmitter();
        this.onParticipantLeft = new core_1.EventEmitter();
        //@Output() onParticipantEvicted: EventEmitter<any> = new EventEmitter();
        //@Output() onUpodateMainSpeaker: EventEmitter<any> = new EventEmitter();
        //@Output() onNewMessage: EventEmitter<any> = new EventEmitter();
        this.onErrorMedia = new core_1.EventEmitter();
        // Next verion: make it an input
        this.toolbarColor = 'primary';
        // Participants
        this.participants = [];
        // Rest of peers
        this.streams = [];
        this.joinedRoom = false;
        this.connected = false;
        // Flags for HTML elements
        this.userMessage = 'Loading OpenViudu...';
        this.micDisabled = false;
        this.cameraDisabled = false;
        this.isFullscreen = false;
    }
    OpenViduComponent.prototype.ngOnInit = function () {
        var _this = this;
        // Set listeners
        window.onbeforeunload = (function () {
            if (_this.openVidu) {
                _this.openVidu.close(true);
            }
        });
        // Display message
        this.userMessage = 'Connecting...';
        // Setup connection to server
        this.openVidu = new openvidu_browser_1.OpenVidu(this.wsUrl);
        this.openVidu.connect(function (error, openVidu) {
            if (error)
                return console.log(error);
            console.log(openVidu);
            _this.connected = true;
            // Display message
            _this.userMessage = 'Joining room...';
            _this.joinSession(_this.sessionId, _this.participantId, function (error) {
                if (error)
                    console.log(error);
                // All correct :)
            });
        });
        // Set listener
        this.bigScreenService.onChange(function () {
            // TODO: check if mainElement is the one with fullscreen
            _this.isFullscreen = _this.bigScreenService.isFullscreen();
        });
    };
    OpenViduComponent.prototype.setListeners = function () {
        var _this = this;
        // Set listeners
        this.session.addEventListener('update-main-speaker', function (participantEvent) {
            console.warn('Update main speaker');
            // TODO.: make a fix in Openvidu ?
            // "participantId" is not a participant ID, is a stream ID
            if (_this.streams === null)
                _this.streams = [];
            for (var _i = 0, _a = _this.streams; _i < _a.length; _i++) {
                var stream = _a[_i];
                console.log(stream.getId());
                if (stream.getId() === participantEvent.participantId) {
                    var realParticipantId = stream.getParticipant().getId();
                    //this.participants.indexOf(oldParticipant)
                    var streams = _this.participants[realParticipantId].getStreams();
                    // Use any stream
                    _this.mainStream = streams[Object.keys(streams)[0]];
                    break;
                }
            }
        });
        this.session.addEventListener('error-room', function () {
            console.warn('error-room');
            // Emit event
            //this.onErrorRoom.emit();
        });
        this.session.addEventListener('room-connected', function () {
            console.warn('Room connected');
            _this.joinedRoom = true;
            // Emit event
            _this.onRoomConnected.emit();
        });
        this.session.addEventListener('stream-added', function (streamEvent) {
            console.warn('Stream added');
            var newStream = streamEvent.stream;
            _this.streams.push(newStream);
            // Also add to participant
            _this.participants[newStream.getParticipant().getId()].addStream(newStream);
        });
        this.session.addEventListener('participant-published', function () {
            console.warn('Participant published');
        });
        this.session.addEventListener('participant-joined', function (participantEvent) {
            console.warn('participant-joined');
            var newParticipant = participantEvent.participant;
            _this.participants[newParticipant.id] = newParticipant;
            // Emit event
            _this.onParticipantJoined.emit({
                participantId: newParticipant.id
            });
        });
        this.session.addEventListener('participant-left', function (participantEvent) {
            console.warn('Participant Left');
            // TODO: manually update main speaker
            var oldParticipant = participantEvent.participant;
            _this.participants.splice(_this.participants.indexOf(oldParticipant), 1);
            // Emit event
            _this.onParticipantLeft.emit({
                participantId: oldParticipant.id
            });
        });
        this.session.addEventListener('stream-removed', function (streamEvent) {
            console.warn('stream-removed');
            // TODO: manually update main speaker
            var oldStream = streamEvent.stream;
            _this.streams.splice(_this.streams.indexOf(oldStream), 1);
        });
        this.session.addEventListener('participant-evicted', function (participantEvent) {
            console.warn('participant-evicted');
            var localParticipant = participantEvent.localParticipant;
            // Emit event
            //this.onParticipantEvicted.emit({
            //	participantId: localParticipant.id
            //});
        });
        this.session.addEventListener('newMessage', function () {
            console.warn('newMessage');
        });
        this.session.addEventListener('room-closed', function () {
            console.warn('room-closed');
            // Emit event
            _this.onRoomClosed.emit();
        });
        this.session.addEventListener('lost-connection', function () {
            console.warn('lost-connection');
            // Emit event
            _this.onLostConnection.emit();
        });
        this.session.addEventListener('error-media', function (errorEvent) {
            console.warn('error-media');
            // Emit event
            _this.onErrorMedia.emit({
                error: errorEvent.error
            });
        });
    };
    OpenViduComponent.prototype.joinSession = function (sessionId, participantId, callback) {
        var _this = this;
        if (!this.connected)
            callback(new Error('Not connected to server yet'));
        // Set attributes
        this.sessionId = sessionId;
        this.participantId = participantId;
        var sessionOptions = {
            sessionId: this.sessionId,
            participantId: this.participantId
        };
        // Enter session
        this.openVidu.joinSession(sessionOptions, function (error, session) {
            if (error)
                return callback(error);
            // Get session for this room
            _this.session = session;
            _this.setListeners();
            var camera = _this.openVidu.getCamera({
                recvAudio: true,
                recvVideo: true,
                audio: true,
                video: true,
                data: true
            });
            // Show camera
            camera.requestCameraAccess(function (error, camera) {
                if (error)
                    return callback(error);
                _this.myCamera = camera;
                _this.myCamera.mirrorLocalStream(_this.myCamera.getWrStream());
                _this.mainStream = _this.myCamera;
                _this.streams.push(_this.myCamera);
                // Debug
                var allStreams = session.getStreams();
                console.log('MY CAMERA APPROVED: ', camera);
                console.log('Other streams: ', allStreams);
                // Publish my camera to session
                _this.myCamera.publish();
                return callback(error);
            });
        });
    };
    OpenViduComponent.prototype.toggleMic = function () {
        var toggleMicButton = this.toggleMicElement._getHostElement();
        if (this.micDisabled) {
            this.renderer.setElementClass(toggleMicButton, 'disabled', false);
            this.myCamera.getWebRtcPeer().audioEnabled = true;
        }
        else {
            this.renderer.setElementClass(toggleMicButton, 'disabled', true);
            this.myCamera.getWebRtcPeer().audioEnabled = false;
        }
        this.micDisabled = !this.micDisabled;
    };
    OpenViduComponent.prototype.toggleCamera = function () {
        var toggleCameraButton = this.toggleCameraElement._getHostElement();
        if (this.cameraDisabled) {
            this.renderer.setElementClass(toggleCameraButton, 'disabled', false);
            this.myCamera.getWebRtcPeer().videoEnabled = true;
        }
        else {
            this.renderer.setElementClass(toggleCameraButton, 'disabled', true);
            this.myCamera.getWebRtcPeer().videoEnabled = false;
        }
        this.cameraDisabled = !this.cameraDisabled;
    };
    OpenViduComponent.prototype.toggleFullscreen = function () {
        if (this.bigScreenService.isFullscreen()) {
            this.bigScreenService.exit();
        }
        else {
            this.bigScreenService.request(this.mainElement.nativeElement);
        }
    };
    /*private onLostConnection() {
        if (!this.joinedRoom) {
            console.warn('Not connected to room, ignoring lost connection notification');
            return false;
        }
    }*/
    OpenViduComponent.prototype.leaveSession = function () {
        this.mainStream = null;
        this.session = null;
        this.streams = [];
        this.participants = [];
        this.userMessage = 'You left the room';
        this.joinedRoom = false;
        if (this.openVidu) {
            this.openVidu.close(true);
        }
    };
    return OpenViduComponent;
}());
OpenViduComponent.decorators = [
    { type: core_1.Component, args: [{
                selector: 'openvidu',
                template: "<div #main class=\"openvidu\">\n\t\n\t<!-- Messages to user -->\n\t<div class=\"message-screen\" *ngIf=\"userMessage && !session\">\n\t\t{{userMessage}}\n\t</div>\n\t\t\n\t<!-- VIDEO CHAT -->\n\t<div class=\"wrapper\" *ngIf=\"session\">\n\n\t\t<!-- TOOLBAR -->\n\t\t<div class=\"toolbar\">\n\t\t\t<md-toolbar [color]=\"toolbarColor\">\n\t\t\t\t<span>{{sessionId}}</span>\n\t\t\t\t<span class=\"example-spacer\"></span>\n\n\t\t\t\t<!-- For fullscreen -->\n\t\t\t\t<button md-icon-button (click)=\"toggleFullscreen()\" *ngIf=\"!isFullscreen\" mdTooltip=\"Fullscreen\" mdTooltipPosition=\"below\">\n\t\t\t\t\t<md-icon>fullscreen</md-icon>\n\t\t\t\t</button>\n\t\t\t\t<button md-icon-button (click)=\"toggleFullscreen()\" *ngIf=\"isFullscreen\" mdTooltip=\"Exit fullscreen\" mdTooltipPosition=\"below\">\n\t\t\t\t\t<md-icon>clear</md-icon>\n\t\t\t\t</button>\n\n\t\t\t\t<!-- RIGHT -->\n\t\t\t\t<!--<button md-icon-button [mdMenuTriggerFor]=\"menu\">\n\t\t\t\t\t<md-icon>more_vert</md-icon>\n\t\t\t\t</button>\n\t\t\t\t<md-menu #menu=\"mdMenu\">\n\t\t\t\t\t<button md-menu-item (click)=\"toggleFullscreen()\">\n\t\t\t\t\t\t<md-icon>fullscreen</md-icon>\n\t\t\t\t\t\t<span>Fullscreen</span>\n\t\t\t\t\t</button>\n\t\t\t\t</md-menu>-->\n\t\t\t</md-toolbar>\n\t\t</div>\n\n\t\t<!-- MAIN VIDEO -->\n\t\t<div class=\"main-video\">\n\t\t\t<div class=\"message\" *ngIf=\"streams.length == 1\">You are alone in this room :(</div>\n\t\t\t<stream class=\"main-stream\" *ngIf=\"mainStream != null && mainStream.getWrStream() != null\" [stream]=\"mainStream\"></stream>\n\t\t\t<div class=\"name-placeholder\">\n\t\t\t\t<div *ngIf=\"mainStream != null\">{{mainStream.getParticipant().id.charAt(0)}}</div>\n\t\t\t\t<div *ngIf=\"mainStream == null && participantId != null\">{{participantId.charAt(0)}}</div>\n\t\t\t</div>\n\t\t</div>\n\n\t\t<!-- BUTTONS & OTHERS VIDEOS -->\n\t\t<div class=\"bottom-bar\">\n\t\t\t<div class=\"buttons\">\n\t\t\t\t<button #toggleMicView md-fab class=\"toggle-mic\" (click)=\"toggleMic()\"><md-icon>mic</md-icon><md-icon class=\"disabled-icon\">mic_off</md-icon></button>\n\n\t\t\t\t<button md-fab class=\"close-session\" (click)=\"leaveSession()\"><md-icon>call_end</md-icon></button>\n\n\t\t\t\t<button #toggleCameraView md-fab class=\"toggle-camera\" (click)=\"toggleCamera()\"><md-icon>videocam</md-icon><md-icon class=\"disabled-icon\">videocam_off</md-icon></button>\n\n\t\t\t</div>\n\t\t\t<div class=\"streams\" *ngIf=\"streams.length > 1\">\n\t\t\t\t<stream class=\"stream side-stream\" *ngFor=\"let s of streams\" [stream]=\"s\" [ngClass]=\"{'hide-video': (s == mainStream)}\">\n\n\t\t\t\t</stream>\n\t\t\t</div>\n\t\t</div>\n\n\t</div>\n<div>\n",
                styles: [".openvidu {\n  width: 100%;\n  height: 100%;\n}\n.openvidu .message-screen {\n  height: 100%;\n  background: #000;\n  color: #fff;\n  font-weight: bold;\n  font-size: 22px;\n  display: -webkit-box;\n  display: -webkit-flex;\n  display: -ms-flexbox;\n  display: flex;\n  -webkit-box-align: center;\n  -webkit-align-items: center;\n      -ms-flex-align: center;\n          align-items: center;\n  -webkit-box-pack: center;\n  -webkit-justify-content: center;\n      -ms-flex-pack: center;\n          justify-content: center;\n}\n.openvidu .wrapper {\n  display: -webkit-box;\n  display: -webkit-flex;\n  display: -ms-flexbox;\n  display: flex;\n  -webkit-box-orient: vertical;\n  -webkit-box-direction: normal;\n  -webkit-flex-direction: column;\n      -ms-flex-direction: column;\n          flex-direction: column;\n  position: relative;\n  height: 100%;\n}\n.openvidu .wrapper .main-video {\n  position: relative;\n  -webkit-box-flex: 1;\n  -webkit-flex: 1;\n      -ms-flex: 1;\n          flex: 1;\n  overflow: hidden;\n  background: #111;\n}\n.openvidu .wrapper .main-video .message {\n  color: #fff;\n  text-align: center;\n  position: absolute;\n  top: 20px;\n  left: 0px;\n  right: 0px;\n  text-shadow: #000 1px 1px 3px;\n  z-index: 99;\n}\n.openvidu .wrapper .main-video .name-placeholder {\n  width: 100%;\n  height: 100%;\n  text-align: center;\n  position: relative;\n}\n.openvidu .wrapper .main-video .name-placeholder div {\n  background: #70826a;\n  width: 160px;\n  height: 160px;\n  border-radius: 50%;\n  left: 50%;\n  top: 50%;\n  position: absolute;\n  margin: -80px 0px 0px -80px;\n  font-size: 100px;\n  color: #fff;\n  line-height: 160px;\n}\n.openvidu .wrapper .toolbar .example-icon {\n  padding: 0 14px;\n}\n.openvidu .wrapper .toolbar .example-spacer {\n  -webkit-box-flex: 1;\n  -webkit-flex: 1 1 auto;\n      -ms-flex: 1 1 auto;\n          flex: 1 1 auto;\n}\n.openvidu .wrapper .bottom-bar {\n  bottom: 0;\n  height: auto;\n  left: 0;\n  position: absolute;\n  text-align: center;\n  right: 0;\n}\n.openvidu .wrapper .bottom-bar .buttons {\n  margin-bottom: 0;\n  text-align: center;\n  white-space: nowrap;\n  position: relative;\n}\n.openvidu .wrapper .bottom-bar .buttons button {\n  color: #fff;\n  background-color: rgba(0, 0, 0, 0.4);\n  margin: 16px 8px 40px 8px;\n}\n.openvidu .wrapper .bottom-bar .buttons button:hover {\n  background-color: rgba(0, 0, 0, 0.6);\n}\n.openvidu .wrapper .bottom-bar .buttons button.toggle-mic.disabled,\n.openvidu .wrapper .bottom-bar .buttons button.toggle-camera.disabled {\n  color: #000;\n  background-color: #ffffff;\n}\n.openvidu .wrapper .bottom-bar .buttons button.toggle-mic.disabled md-icon,\n.openvidu .wrapper .bottom-bar .buttons button.toggle-camera.disabled md-icon {\n  display: none;\n}\n.openvidu .wrapper .bottom-bar .buttons button.toggle-mic.disabled md-icon.disabled-icon,\n.openvidu .wrapper .bottom-bar .buttons button.toggle-camera.disabled md-icon.disabled-icon {\n  display: inline-block;\n}\n.openvidu .wrapper .bottom-bar .buttons button.toggle-mic md-icon,\n.openvidu .wrapper .bottom-bar .buttons button.toggle-camera md-icon {\n  display: inline-block;\n}\n.openvidu .wrapper .bottom-bar .buttons button.toggle-mic md-icon.disabled-icon,\n.openvidu .wrapper .bottom-bar .buttons button.toggle-camera md-icon.disabled-icon {\n  display: none;\n}\n.openvidu .wrapper .bottom-bar .buttons button.close-session {\n  background-color: #db4437;\n}\n.openvidu .wrapper .bottom-bar .streams {\n  display: -webkit-box;\n  display: -webkit-flex;\n  display: -ms-flexbox;\n  display: flex;\n  -webkit-box-pack: start;\n  -webkit-justify-content: flex-start;\n      -ms-flex-pack: start;\n          justify-content: flex-start;\n  max-width: 100%;\n  text-align: start;\n  padding: 8px 24px 0 24px;\n  height: 132px;\n  white-space: nowrap;\n  -webkit-transition: bottom 250ms ease-out, height 250ms ease-out;\n  transition: bottom 250ms ease-out, height 250ms ease-out;\n  -webkit-box-sizing: border-box;\n  box-sizing: border-box;\n  direction: rtl;\n  overflow: auto;\n  overflow: -moz-scrollbars-none;\n}\n"],
                providers: [angular_bigscreen_1.BigScreenService],
                encapsulation: core_1.ViewEncapsulation.None
            },] },
];
/** @nocollapse */
OpenViduComponent.ctorParameters = function () { return [
    { type: core_1.Renderer, },
    { type: angular_bigscreen_1.BigScreenService, },
]; };
OpenViduComponent.propDecorators = {
    'wsUrl': [{ type: core_1.Input },],
    'sessionId': [{ type: core_1.Input },],
    'participantId': [{ type: core_1.Input },],
    'onRoomConnected': [{ type: core_1.Output },],
    'onRoomClosed': [{ type: core_1.Output },],
    'onLostConnection': [{ type: core_1.Output },],
    'onParticipantJoined': [{ type: core_1.Output },],
    'onParticipantLeft': [{ type: core_1.Output },],
    'onErrorMedia': [{ type: core_1.Output },],
    'mainElement': [{ type: core_1.ViewChild, args: ['main',] },],
    'toggleMicElement': [{ type: core_1.ViewChild, args: ['toggleMicView',] },],
    'toggleCameraElement': [{ type: core_1.ViewChild, args: ['toggleCameraView',] },],
};
exports.OpenViduComponent = OpenViduComponent;
//# sourceMappingURL=openvidu.component.js.map